﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Telerik.Windows.DragDrop;
using System.Collections;

namespace DragAndDropManager
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            ApplicationList.ItemsSource = ApplicationInfo.GenerateApplicationInfos();
            MyAppList.ItemsSource = new ObservableCollection<ApplicationInfo>();

            DragDropManager.AddDragInitializeHandler(ApplicationList, OnDragInitialize);
            DragDropManager.AddDragInitializeHandler(MyAppList, OnDragInitialize);

            DragDropManager.AddGiveFeedbackHandler(ApplicationList, OnGiveFeedback);
            DragDropManager.AddGiveFeedbackHandler(MyAppList, OnGiveFeedback);

            DragDropManager.AddDragDropCompletedHandler(ApplicationList, OnDragCompleted);
            DragDropManager.AddDragDropCompletedHandler(MyAppList, OnDragCompleted);

            DragDropManager.AddDropHandler(ApplicationList, OnDrop);
            DragDropManager.AddDropHandler(MyAppList, OnDrop);
        }

        private void OnDragInitialize(object sender, DragInitializeEventArgs args)
        {
            args.AllowedEffects = DragDropEffects.All;
            var payload = DragDropPayloadManager.GeneratePayload(null);
            payload.SetData(typeof(ApplicationInfo), ((FrameworkElement)args.OriginalSource).DataContext);
            args.Data = payload;
            
            args.DragVisual = new ContentControl
            { 
                Content = args.Data, ContentTemplate = LayoutRoot.Resources["ApplicationTemplate"] as DataTemplate
            };
        }

        private void OnGiveFeedback(object sender, Telerik.Windows.DragDrop.GiveFeedbackEventArgs args)
        {
            args.SetCursor(Cursors.Arrow);
            args.Handled = true;
        }

        private void OnDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs args)
        {
            var data = args.Data as IDataObject;
            var appInfo = data.GetData(typeof(ApplicationInfo)) as ApplicationInfo;
            ((IList)(sender as ListBox).ItemsSource).Add(appInfo);
        }

        public void OnDragCompleted(object sender, Telerik.Windows.DragDrop.DragDropCompletedEventArgs args)
        {
            var data = args.Data as IDataObject;
            var appInfo = data.GetData(typeof(ApplicationInfo)) as ApplicationInfo;
            ((IList)(sender as ListBox).ItemsSource).Remove(appInfo);
        }
    }
}
